#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/base/prototype.rb'
# include ::FtpParadise::Base
# =========================================================================== #
require 'ftp_paradise/constants/constants.rb'
# =========================================================================== #
# The next .rb file will pull in "cliner":
# =========================================================================== #
require 'ftp_paradise/requires/common_external_requires.rb'

module FtpParadise

module Base

  # ========================================================================= #
  # The various project's toplevel_methods files are required next:
  # ========================================================================= #
  require 'ftp_paradise/requires/require_the_toplevel_methods.rb'

  # =========================================================================== #
  # Colour support comes next:
  # =========================================================================== #
  require 'ftp_paradise/base/colours.rb'
  require 'ftp_paradise/base/cliner.rb'
  require 'ftp_paradise/base/reset.rb'

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === be_verbose?
  # ========================================================================= #
  def be_verbose?
    @internal_hash[:be_verbose]
  end

  # ========================================================================= #
  # === be_quiet
  # ========================================================================= #
  def be_quiet
    @internal_hash[:be_verbose] = false
  end; alias be_silent be_quiet # === be_silent

  # ========================================================================= #
  # === set_be_verbose
  #
  # Set the verbosity mode.
  # ========================================================================= #
  def set_be_verbose(i = true)
    @internal_hash[:be_verbose] = i
  end; alias be_verbose= set_be_verbose # === be_verbose=
       alias be_verbose  set_be_verbose # === be_verbose

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn(i = NAMESPACE)
    if i.is_a? String
      i = { namespace: i }
    end
    FtpParadise.opnn(i)
  end

  # ========================================================================= #
  # === rds
  # ========================================================================= #
  def rds(i = '')
    FtpParadise.rds(i)
  end

  # ========================================================================= #
  # === touch
  # ========================================================================= #
  def touch(i = '')
    FtpParadise.touch(i)
  end

  # ========================================================================= #
  # === get_files
  # ========================================================================= #
  def get_files(i = '')
    FtpParadise.get_files(i)
  end

  # ========================================================================= #
  # === return_all_local_entries
  #
  # This will return an array.
  # ========================================================================= #
  def return_all_local_entries(
      from_where = Dir.pwd
    )
    Dir["#{from_where}/**/**"]
  end; alias return_directory_content return_all_local_entries # === return_directory_content

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i = '')
    FtpParadise.e(i)
  end

  # ========================================================================= #
  # === use_colours?
  # ========================================================================= #
  def use_colours?
    @internal_hash[:use_colours]
  end

  # ========================================================================= #
  # === parens                                                   (parens tag)
  # ========================================================================= #
  def parens(i)
    padding_to_use = ' ' *  5
    return sfancy("#{padding_to_use}(#{i}) ")
  end

  # ========================================================================= #
  # === pound_token
  #
  # Simply return a colourized pound token next, but only if we use colours.
  # ========================================================================= #
  def pound_token
    _ = '# '
    if use_colours?
      return "#{Colours::CYAN}#{_}#{Colours::WHITE}"
    end
    _
  end

  # ========================================================================= #
  # === on_roebe?
  # ========================================================================= #
  def on_roebe?
    ENV['IS_ROEBE'].to_s == '1'
  end; alias are_we_on_roebe? on_roebe? # === are_we_on_roebe?

  # ========================================================================= #
  # === return_time
  # ========================================================================= #
  def return_time
    FtpParadise.return_time
  end

  # ========================================================================= #
  # === do_use_colours
  # ========================================================================= #
  def do_use_colours
    @internal_hash[:use_colours] = true
  end

  # ========================================================================= #
  # === cd
  # ========================================================================= #
  def cd(i)
    Dir.chdir(i) if File.directory? i
  end; alias change_local_directory cd # === change_local_directory

  # ========================================================================= #
  # === return_pwd
  # ========================================================================= #
  def return_pwd
    ::FtpParadise.return_pwd.to_s
  end; alias return_current_directory return_pwd # === return_current_directory

end; end