#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen-string-literal: true
# =========================================================================== #
# This file will add colour-related methods to the ftp_paradise project.
# =========================================================================== #
# require 'ftp_paradise/colours/colours.rb'
# =========================================================================== #
module FtpParadise

  require 'ftp_paradise/colours/use_colours.rb'

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    return Colours.rev if use_colours?
    return ''
  end

  # ========================================================================= #
  # === FtpParadise.sdir
  # ========================================================================= #
  def self.sdir(i)
    return Colours.sdir(i) if use_colours?
    i
  end

  # ========================================================================= #
  # === FtpParadise.swarn
  # ========================================================================= #
  def self.swarn(i)
    return Colours.swarn(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === FtpParadise.sfile
  # ========================================================================= #
  def self.sfile(i)
    return Colours.sfile(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === FtpParadise.sfancy
  # ========================================================================= #
  def self.sfancy(i)
    return Colours.sfancy(i) if use_colours? # Else we won't modify this at all.
    return i
  end

  # ========================================================================= #
  # === FtpParadise.simp
  # ========================================================================= #
  def self.simp(i)
    return Colours.simp(i) if use_colours? # Else we won't modify this.
    return i
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i)
    FtpParadise.sfancy(i) # We can delegate here, because FtpParadise.sfancy() will take care of colours.
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i) # Link to FtpParadise.simp
    FtpParadise.simp(i) # We can delegate here, because FtpParadise.sfancy will take care of colours.
  end

  # ========================================================================= #
  # === sdir
  # ========================================================================= #
  def sdir(i)
    FtpParadise.sdir(i)
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i)
    FtpParadise.sfile(i)
  end

  # ========================================================================= #
  # === swarn
  # ========================================================================= #
  def swarn(i)
    FtpParadise.swarn(i)
  end

  # ========================================================================= #
  # === FtpParadise.lightyellow
  # ========================================================================= #
  def self.lightyellow(i = '')
    if use_colours?
      return ::Colours.lightyellow(i)
    end
    return "#{Colours.yellow}#{i}#{Colours.rev}"
  end; self.instance_eval { alias yellow lightyellow } # === FtpParadise.lightyellow

  # ========================================================================= #
  # === FtpParadise.royalblue
  # ========================================================================= #
  def self.royalblue(i = '')
    if use_colours?
      return ::Colours.royalblue(i)
    end
    return i
  end

  # ========================================================================= #
  # === FtpParadise.slateblue
  # ========================================================================= #
  def self.slateblue(i = '')
    if use_colours?
      return ::Colours.slateblue(i)
    end
    return i
  end

  # ========================================================================= #
  # === FtpParadise.testing_colours          (colour test, testing colours)
  #
  # This just is a test whether we use colours or whether we won't.
  # ========================================================================= #
  def self.testing_colours
    e 'We will now test some colours.'
    e "Will we use colours? #{FtpParadise.use_colours?.to_s}"
    e 'Testing '+sfancy('sfancy')+' next.'
  end

end

if __FILE__ == $PROGRAM_NAME
  puts FtpParadise.swarn('Oh noes!')
end