#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/colours/use_colours.rb'
# =========================================================================== #
module FtpParadise

  require 'ftp_paradise/configuration/configuration.rb'

  begin
    require 'colours'
  rescue LoadError; end

  # ========================================================================= #
  # === @configuration
  #
  # Let's initialize the configuration here, if we have not yet
  # done so.
  # ========================================================================= #
  if @configuration.nil? and FtpParadise.respond_to?(:initialize_configuration)
    FtpParadise.initialize_configuration
  end

  # ========================================================================= #
  # === FtpParadise.use_colours?
  #
  # This can be delegated towards the configuration object.
  # ========================================================================= #
  def self.use_colours?
    if FtpParadise.respond_to?(:config?) and
       FtpParadise.config?.respond_to?(:use_colours)
      FtpParadise.config?.use_colours
    else
      true
    end
  end

  # ========================================================================= #
  # === FtpParadise.disable_colours
  #
  # Use this method to disable colours.
  # ========================================================================= #
  def self.disable_colours
    FtpParadise.config?.use_colours = false
  end

  # ========================================================================= #
  # === use_colours?
  #
  # Whether we will use colours or whether we will not.
  # ========================================================================= #
  def use_colours?
    FtpParadise.use_colours?
  end

  # ========================================================================= #
  # === If we use colours in this project
  # ========================================================================= #
  if use_colours?
    # ======================================================================= #
    # And we can also set the default-colours to use.
    # ======================================================================= #
    if Object.const_defined?(:Colours) and
       Colours.respond_to?(:assign_new_colour_codes)
      Colours.update_the_colour_codes_with({
        default_colour: :green,
        sfile:          :brown,
        sdir:           :chocolate,
        swarn:          :red,
        simportant:     :bold_yellow,
      })
    end
  end

end