#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen-string-literal: true
# =========================================================================== #
# require 'ftp_paradise/configuration/configuration.rb'
# =========================================================================== #
module FtpParadise

  require 'ftp_paradise/project/project.rb'

  begin # Try to make use of Roebe::Configuration next.
    require 'roebe/configuration/configuration.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === @configuration
  # ========================================================================= #
  @configuration = nil

  # ========================================================================= #
  # === FtpParadise.initialize
  #
  # This method will assign to the general configuration object stored
  # in the variable @configuration.
  # ========================================================================= #
  def self.initialize
    if Object.const_defined? :Roebe
      # ===================================================================== #
      # Load up all .yml files from the path to the yaml directory:
      # ===================================================================== #
      @configuration = ::Roebe::Configuration::Configuration.new(FtpParadise.yaml_directory?)
    end
  end; self.instance_eval { alias initialize_configuration initialize } # === FtpParadise.initialize_configuration

  # ========================================================================= #
  # === Initialize the Configuration object here
  # ========================================================================= #
  FtpParadise.initialize_configuration if @configuration

  # ========================================================================= #
  # === FtpParadise.config?
  #
  # Query method towards the Configuration object.
  # ========================================================================= #
  def self.config?
    @configuration
  end; self.class { alias configuration? config? } # === FtpParadise.configuration?

end