#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This is the "connection class". It will have methods that can be use
# when we are connected to a remote site (via the FTP protocol).
#
# Rather than keeping FTP-related state within this class, such as remote
# URL and such, we will instead use FtpParadise module methods.
# =========================================================================== #
# Possible errors and failures when we are trying to connect:
#
#   Net::FTPPermError: 530 Please login with USER and PASS.
#
#   Net::FTPConnectionError is raised if we are no longer connected to
#   the remote FTP site.
#
#   Errno::ETIMEDOUT is raised when a timeout event occurred.
# =========================================================================== #
# The official documentation for Ruby-FTP can be found here:
#
#   https://ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html
#
# =========================================================================== #
require 'ftp_paradise/connection/initialize.rb'
require 'ftp_paradise/connection/data.rb'
require 'ftp_paradise/connection/debug.rb'
require 'ftp_paradise/connection/misc.rb'
require 'ftp_paradise/connection/notify.rb'
require 'ftp_paradise/connection/remote_pwd.rb'
require 'ftp_paradise/connection/remove.rb'
require 'ftp_paradise/connection/show.rb'
require 'ftp_paradise/connection/upload.rb'
require 'ftp_paradise/version/version.rb'
require 'ftp_paradise/configuration/configuration.rb'