#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/data.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/constants/roebe.rb'

  # ========================================================================= #
  # === set_data                                   (all in one, set_data tag)
  #
  # This method sets:
  #
  #   - host
  #   - user name
  #   - port and
  #   - password
  #
  # This method is thus a nifty little convenience method. You can pass
  # a hash to it too, which is the recommended way to use this method.
  #
  # You can also use an Array (with 4 elements), in which case we will
  # process it one after the other. The reason for this procedure is
  # because we can also use yaml files that way, that also use the
  # exact order of elements.
  #
  # If you add a new entry to the case menu below, also add this to 
  # ARRAY_AVAILABLE_HOSTS, please. That Array can be found in the
  # file 'constants/roebe_ftp_constants.rb'.
  # ========================================================================= #
  def set_data(
      i = 'default'
    ) # set the default here.
    if debug?
      opnn; e 'We will debug next, as the @debug variable was set to true.'
      pp i
    end
    # ======================================================================= #
    # === Handle Hash as input first
    # ======================================================================= #
    if i.is_a? Hash # Hash is treated differently.

      # ===================================================================== #
      # === :host
      # ===================================================================== #
      if   i.has_key? :host
        set_host i.fetch(:host)
      elsif i.has_key? :remote_host
        set_host i.fetch(:remote_host)
      elsif i.has_key? :to # :to is an alias to :host.
        set_host i.fetch(:to)
      end      

      # ===================================================================== #
      # === :user_name
      # ===================================================================== #
      if    i.has_key? :user_name
        set_user_name i.fetch(:user_name)
      elsif i.has_key? :login_name
        set_user_name i.fetch(:login_name)
      end

      # ===================================================================== #
      # === :password
      # ===================================================================== #
      if i.has_key? :password
        set_password i.fetch(:password)
      elsif i.has_key? :login_password
        set_password i.fetch(:login_password)
      end

      # ===================================================================== #
      # === :port
      # ===================================================================== #
      if i.has_key? :port
        set_port i.fetch(:port)
      end
    # ======================================================================= #
    # === Handle Array as input next
    # ======================================================================= #
    elsif i.is_a? Array # Example-Array: ["ftp.byethost33.com", "b33_14144659", "1aaaaaa", 21]
      set_host      i.first
      set_user_name i[1]
      set_password  i[2]
      set_port      i[3]
    # ======================================================================= #
    # === Else, use simple toggle commands, such as 1,2,3,4,5 and so forth.
    # ======================================================================= #
    else # In the following case menu you can toggle your FTP.
      case i.to_s # Work on strings only. Assumes shortcuts.
      # ===================================================================== #
      # === shevy
      #
      # This defaults to my home system.
      # ===================================================================== #
      when '6',
           /shevy/i,
           'square',
           'square7',
           'bplaced',
           'podserver',
           'default',
           'top' # Current default.
        _ = FILE_ROEBE_FTP
        if File.exist? _
          data = YAML.load_file(_)['podserver'] # 'bplaced'
          set_host      data['url']
          set_user_name data['user_name']
          this_password = data['password']
          this_password.chop! if this_password.end_with? '_'
          set_password  this_password
          set_port      data['port']
        end
      # ===================================================================== #
      # === a1
      # ===================================================================== #
      when '1',
           'a1',
           'a1_ftp' # A1 FTP host. No longer in use but still kept.
        set_host      A1.first
        set_user_name A1[1]
        set_password  A1[2]
        set_port      A1[3]
      # ===================================================================== #
      # === zymix
      # ===================================================================== #
      when '2',
           'zymix',
           'standard'
        set_host      ZYMIX.first
        set_user_name ZYMIX[1]
        set_password  ZYMIX[2]
        set_port      ZYMIX[3]
      # ===================================================================== #
      # === uniwien
      # ===================================================================== #
      when '3',
           'univie',
           'uniwien' # Uni Wien FTP Account.
        set_host      UNIVIE.first
        set_user_name UNIVIE[1]
        set_password  UNIVIE[2]
        set_port      UNIVIE[3]
      # ===================================================================== #
      # === byte
      # ===================================================================== #
      when '4',
           'byte',
           'bytehost'
        set_host      BYTEHOST.first
        set_user_name BYTEHOST[1]
        set_password  BYTEHOST[2]
        set_port      BYTEHOST[3]
      else
        set_data(:default) # Go back to default again if not found.
      end
    end
  end

end; end