#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/debug.rb'
# =========================================================================== #
module FtpParadise

class Connection

  # ========================================================================= #
  # === set_debug
  # ========================================================================= #
  def set_debug(i = SHALL_WE_DEBUG)
    @internal_hash[:debug] = i
  end

  # ========================================================================= #
  # === enable_debug_mode
  #
  # When the debug mode is set to true, then all traffic to and from
  # the server is written to $stdout. By default, this will NOT
  # be the case.
  # ========================================================================= #
  def enable_debug_mode
    ftp_object?.debug_mode = true
  end

  # ========================================================================= #
  # === enable_debug
  #
  # Set the @debug variable to true, and also invoke enable_debug_mode().
  # ========================================================================= #
  def enable_debug
    enable_debug_mode
    set_debug(true)
  end

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    @internal_hash[:debug]
  end

  # ========================================================================= #
  # === disable_debug
  # ========================================================================= #
  def disable_debug
    ftp_object?.debug_mode = false
    @internal_hash[:debug] = false
  end

  # ========================================================================= #
  # === debug                                                     (debug tag)
  #
  # This method can be used for some debugging output.
  # ========================================================================= #
  def debug(
      shall_we_debug = debug?
    )
    cliner {
      pp ftp?
      show_host_user_name_port_and_password
      update_file_listing
      if shall_we_debug
        opnn; e 'File Listing:'
        show_file_listing
      end
      if remote_file_listing?.empty?
        opnn; e 'No file could be found. The remote directory '\
                'is most likely empty.'
      end
    } if shall_we_debug
  end; alias feedback debug # === feedback
       alias report   debug # === report

end; end