#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/download.rb'
# =========================================================================== #
module FtpParadise

class Connection

  # ========================================================================= #
  # === download_binary_file
  #
  # Use this method to download a binary file, by calling the method
  # getbinaryfile(). You can download as many files as you want to
  # through this method, but we expect this method to be a binary
  # file.
  #
  # Specific usage example:
  #
  #   ftp?.download_binary_file('photos_2009-03-29.zip', 'ftp_photos/photos.zip')
  #
  # ========================================================================= #
  def download_binary_file(
      i,
      be_verbose = be_verbose?
    )
    if i.is_a? Array
      i.each {|entry| download_binary_file(entry) }
    else
      ftp?.getbinaryfile(i) # Delegate towards .getbinaryfile()
      if be_verbose
        e "Downloaded file #{sfile(i)}."
      end
    end
  end; alias getbinaryfile download_binary_file # === getbinaryfile

  # ========================================================================= #
  # === download_remote_file                                   (download tag)
  #
  # Use this method to download a remote file. If you are sure to have
  # a binary file, the method download_binary_file() could be used
  # instead.
  #
  # gettextfile(remotefile, localfile = File.basename(remotefile)) {|line| ...}
  # ========================================================================= #
  def download_remote_file(
      i, be_verbose = be_verbose?
    )
    if i.is_a? Array # Handle arrays first.
      i.each {|entry| download_remote_file(entry, be_verbose) }
    else
      if i =~ /^\d+$/ # if is a number
        update_file_listing
        i = return_remote_files[i.to_i - 1].first
      end
      if i == '*' # Here we want to download all files, as we passed in '*'.
        i = []
        update_file_listing
        return_remote_files.each { |a,b| i << FtpParadise.rds(a) }
      end
      if be_verbose
        notify_the_user_about(i, __method__)
      end
      set_file(i) # This assigns to @internal_hash[:file].
      begin # We can use @file because we called set_file() before.
        ftp_object?.gettextfile(file?)
        basename = File.basename(file?)
        e "Finished downloading `#{sandybrown(file?)}`"
        e 'into `'+sdir(rds(Dir.pwd+'/'+basename))+'`.'
        return file?
      rescue Net::FTPPermError => error
        pp error
        e 'An exception occurred in the method '+sfancy('download()')+'.'
        e 'The error was a '+sfancy('Permission Error')+', the input '\
          'to the method was: `'+simp(i)+'`.'
        e 'Is the file that you are trying to download really '\
          'existing on the remote server?'
        return false
      end
    end
  end; alias download                  download_remote_file # === download
       alias download_this_remote_file download_remote_file # === download_this_remote_file
       alias download_this_file        download_remote_file # === download_this_file

end; end