#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/initialize.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/base/prototype.rb'
  include ::FtpParadise::Base

  require 'ftp_paradise/connection/constants.rb' # ← For the constant RUN_ALREADY
  require 'ftp_paradise/connection/ftp_object.rb'
  require 'ftp_paradise/connection/reset.rb'
  require 'ftp_paradise/connection/run.rb'
  require 'ftp_paradise/connection/set_input.rb'
  require 'ftp_paradise/connection/remote_url.rb'
  require 'ftp_paradise/connection/use_default_dataset.rb'

  # ========================================================================= #
  # === initialize
  #
  # The first argument to this method should be the remote host's URL.
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = RUN_ALREADY
    )
    reset
    # ======================================================================= #
    # - If we pass in a Hash as the first argument, then we will use
    #   set_data(), otherwise we will set individually it all individually.
    # ======================================================================= #
    if i.is_a? Hash
      set_data(i)
    else
      set_input(i)
    end
    case i
    when :dont_run_yet,
         :do_not_run_yet
      run_already = false
    end
    case run_already
    when :dont_run_yet
      run_already = false
    end
    if run_already.is_a? Hash
      _ = run_already
      if _.has_key? :connect_to_this_host
        set_this_host(
          _.delete(:connect_to_this_host)
        )
      end
      run_already = RUN_ALREADY # Restore to the default in this case.
    end
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        run_already = false
      # ===================================================================== #
      # The "default" dataset defaults to my personal FTP login dataset.
      # ===================================================================== #
      when :use_default_dataset,
           :default_dataset,
           :default
        use_default_dataset
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === initialize_a_new_net_ftp_object_with_this_remote_url
  #
  # This method is the only one allowed to create a new Net::FTP instance.
  # ========================================================================= #
  def initialize_a_new_net_ftp_object_with_this_remote_url(i)
    @ftp_object = Net::FTP.new(i) # ftp = Net::FTP.new(YAML.load_file(ENV['FTP_YAML_FILE'])['freehosting']['url'])
  end; alias initialize_a_new_net_ftp_object initialize_a_new_net_ftp_object_with_this_remote_url # === initialize_a_new_net_ftp_object

end; end