#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/is_connected.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/connection/ftp_object.rb'

  # ========================================================================= #
  # === closed?
  #
  # Returns true if the connection to our remote host is closed.
  # ========================================================================= #
  def closed?
    begin
      ftp_object?.closed? if ftp_object?
    rescue Net::FTPConnectionError
      true
    end
  end

  # ========================================================================= #
  # === open?
  #
  # This method can also be used to determine whether we are still
  # connected to the remote site.
  #
  # This method will return true if the connection is open.
  #
  # If you want to determine whether you are still connected to the
  # remote host, you can alias use the alias are_we_connected? - it
  # may read nicer.
  # ========================================================================= #
  def open?
    !closed?
  end; alias are_we_connected? open? # === are_we_connected?
       alias is_connected?     open? # === is_connected?
       alias is_connected      open? # === is_connected
       alias connected?        open? # === connected?
       alias did_we_connect?   open? # === did_we_connect?

end; end