#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/remote_pwd.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/connection/remote_url.rb'

  # ========================================================================= #
  # === remote_pwd
  # ========================================================================= #
  def remote_pwd
    ftp_object?.pwd
  end; alias rpwd   remote_pwd # === rpwd
       alias getdir remote_pwd # === getdir

  # ========================================================================= #
  # === return_remote_pwd
  #
  # This method will return the (remote) pwd, including the remote working
  # directory - including the subpath..
  #
  # This method will ensure that a trailing '/' will be returned here.
  #
  # The official documentation can be seen here:
  #   https://ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html#method-i-pwd
  # ========================================================================= #
  def return_remote_pwd
    begin
      if is_connected?
        remote_pwd = remote_url?.dup
        # ===================================================================== #
        # Need to safeguard, in the event that we are not connected to a
        # remote FTP server.
        # ===================================================================== #
        remote_pwd << ftp_object?.pwd.to_s if ftp_object?
        remote_pwd << '/'
        return rds(remote_pwd)
      else
        opnn; e 'No FTP connection is open.'
        ''
      end
    rescue Net::FTPReplyError
      opnn; e 'Error Net::FTPReplyError received. May be due to user interrupting.'
      opnn; e 'Exiting now at once either way.'
      exit
    rescue SystemExit, Interrupt
      opnn; e 'User requested to exit, thus exiting now.'
      exit
    end
  end; alias pwd?                return_remote_pwd # === pwd?
       alias pwd                 return_remote_pwd # === pwd (pwd tag)
       alias getdir              return_remote_pwd # === get_dir
       alias remote_pwd          return_remote_pwd # === remote_pwd
       alias remote_pwd?         return_remote_pwd # === remote_pwd?
       alias connected_to?       return_remote_pwd # === connected_to?
       alias remote?             return_remote_pwd # === remote?
       alias base_dir?           return_remote_pwd # === base_dir?
       alias remote_directory?   return_remote_pwd # === remote_directory?
       alias remote_dir?         return_remote_pwd # === remote_dir?
       alias remote_path?        return_remote_pwd # === remote_path?
       alias return_subdirectory return_remote_pwd # === return_subdirectory
       alias current_directory   return_remote_pwd # === current_directory
       alias return_remote_dir   return_remote_pwd # === return_remote_dir
       alias get_remote_dir      return_remote_pwd # === get_remote_dir
       alias remote_host?        return_remote_pwd # === remote_host?

end; end