#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/remote_url.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/toplevel_methods/remote_url.rb'

  # ========================================================================= #
  # === remote_url?
  #
  # This method is guaranteed to return a String.
  # ========================================================================= #
  def remote_url?
    begin
      return FtpParadise.remote_url?.to_s
    rescue Interrupt
      exit # User interrupted, so we exit as requested.
    rescue Exception => error
      opnn; e 'We did encounter an error in the method '+
              simp(__method__.to_s)
      opnn; pp error
      return error.to_s # Return that exception.
    end
  end; alias remote_host? remote_url? # === remote_host?
       alias remote_url   remote_url? # === remote_url
       alias url?         remote_url? # === url?
       alias host?        remote_url? # === host?
       alias host         remote_url? # === host

  # ========================================================================= #
  # === set_remote_url
  #
  # This method will do some sanitizing before it sets a proper and
  # valid hostname. Remember to keep the 'default' entry at the
  # current main FTP host you use.
  # ========================================================================= #
  def set_remote_url(
      i          = :default,
      be_verbose = true
    )
    i = :default? if i.nil?
    if i.is_a? Hash # Handle Hash as input here.
      if i.has_key?(:to)
        i = i[:to]
      end
    end
    # ======================================================================= #
    # The next block is mostly for my custom constants.
    # You can simply pass in your own Hash.
    # ======================================================================= #
    case i # Do some sanitizing. This is a String here. (case tag)
    # ======================================================================= #
    # === shevy
    # ======================================================================= #
    when /^-?-?shevy$/i,
         'default'
      set_data(PODSERVER)
      i = PODSERVER.first
    # ======================================================================= #
    # === bplaced
    # ======================================================================= #
    when /^-?-?bplaced$/i
      set_data(BPLACED)
      i = BPLACED.first
    # ======================================================================= #
    # === ?
    # ======================================================================= #
    when '?'
      feedback_login_hosts
    # ======================================================================= #
    # === uniwien_homepage
    # ======================================================================= #
    when /^uniwien_?homepage/
      set_data(UNIWIEN_HOMEPAGE)
      i = UNIWIEN_HOMEPAGE.first
    # ======================================================================= #
    # === unet
    # ======================================================================= #
    when 'unet','uni',
         'uniftp'
      set_data(UNIVIE)
      i = UNIVIE.first   
    # ======================================================================= #
    # === a1
    # ======================================================================= #
    when /^a1/
      set_data(A1)
      i = A1.first
    # ======================================================================= #
    # === byte
    # ======================================================================= #
    when /^byte/ # This also includese bytehost.
      set_data(BYTEHOST)
      i = BYTEHOST.first
    # ======================================================================= #
    # === zerofees
    # ======================================================================= #
    when 'ZEROFEES','zero','ZERO'
      set_data(ZEROFEES)
      i = ZEROFEES.first
    # ======================================================================= #
    # === ucoz
    # ======================================================================= #
    when 'ucoz','UCOZ','LAST',/tanriel/i,
         'last'
      set_data(UCOZ)
      i = UCOZ.first
    end
    case i.to_s
    # ======================================================================= #
    # === shevy
    # ======================================================================= #
    when 'shevy','5','default' # Current default entry.
      i = SQUARE7.first
    # ======================================================================= #
    # === a1
    # ======================================================================= #
    when '1','a1'
      i = A1.first
    # ======================================================================= #
    # === unet
    # ======================================================================= #
    when '2','unet','uni','uniftp'
      i = UNIVIE.first
    # ======================================================================= #
    # === geas
    # ======================================================================= #
    when '3','geas'
      i = GEAS.first
    # ======================================================================= #
    # === byte
    # ======================================================================= #
    when '4','byte'
      i = BYTEHOST.first
    end
    i = i.to_s # Need a String here.
    # ======================================================================= #
    # === Provide more information to the user
    # ======================================================================= #
    if debug?
      opnn; e "The remote URL was set to: #{sfancy(i)}"
    end
    if be_verbose
      e "Setting url to #{sfancy(i)} now."
    end
    FtpParadise.set_remote_url(i)
    # ======================================================================= #
    # And we will always initialize a new ftp-object as well, whenever
    # we call this method:
    # ======================================================================= #
    initialize_a_new_net_ftp_object_with_this_remote_url(i)
  end; alias set_this_host   set_remote_url # === set_this_host
       alias set_host        set_remote_url # === set_host
       alias set_url         set_remote_url # === set_url
       alias host=           set_remote_url # === host=
       alias url=            set_remote_url # === url=
       alias set_remote_host set_remote_url # === set_remote_host

end; end