#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/reset.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/connection/port.rb'
  require 'ftp_paradise/connection/set_array_available_hosts.rb'
  require 'ftp_paradise/connection/transfer_mode.rb'
  require 'ftp_paradise/connection/username.rb'

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @input
    # ======================================================================= #
    @input = nil
    # ======================================================================= #
    # === @ftp_object
    # ======================================================================= #
    @ftp_object = nil
    populate_internal_hash_with_default_values
    # ======================================================================= #
    # The next method-call must come after
    # populate_internal_hash_with_default_values().
    # ======================================================================= #
    set_be_verbose if BE_VERBOSE
    set_port
    set_array_available_hosts
    set_default_transfer_mode :be_quiet
    set_username
  end

  # ========================================================================= #
  # === populate_internal_hash_with_default_values
  # ========================================================================= #
  def populate_internal_hash_with_default_values
    # ======================================================================= #
    # === @internal_hash
    # ======================================================================= #
    @internal_hash = {}
    # ======================================================================= #
    # === :port
    #
    # The internal Hash keeps track of some configure-settings and slots
    # to be used that are of relevance to this class.
    # ======================================================================= #
    @internal_hash[:port] = nil     # Has to be nil initially.
    @internal_hash[:be_verbose] = nil
    @internal_hash[:raw_entries] = nil
    @internal_hash[:file] = nil
    @internal_hash[:transfer_mode] = :binary
    @internal_hash[:show_full_names] = false # Is false on startup.
    @internal_hash[:debug] = false
    # ======================================================================= #
    # === :array_available_hosts
    # ======================================================================= #
    @internal_hash[:array_available_hosts] = []
    # ======================================================================= #
    # === :padding
    # ======================================================================= #
    @internal_hash[:padding] = '  ' # This padding value is used for remote directory listing.
    # ======================================================================= #
    # The following Array keeps track over the files that were uploaded 
    # successfully. This gives us the possibility to query this from other
    # classes.
    # ======================================================================= #
    @internal_hash[:uploaded_these_files] = []
  end

end; end