#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === FtpParadise::GUI::Gtk::FtpParadiseModule
#
# The purpose of this class is to provide a GTK-FTP Client, loosely
# modelled after the client called "gftp".
# =========================================================================== #
# require 'ftp_paradise/gui/shared_code/ftp_paradise/ftp_paradise_module.rb'
# include ::FtpParadise::GUI::Gtk::FtpParadiseModule
# =========================================================================== #
module FtpParadise

module GUI

module Gtk

module FtpParadiseModule

  require 'ftp_paradise/base/prototype.rb'
  include ::FtpParadise::Base

  require 'net/ftp'

  require 'ftp_paradise/project/project.rb'

  require 'ftp_paradise/constants/roebe_ftp_constants.rb'

  require 'gtk_paradise/widgets/gtk3/directory_content_widget/directory_content_widget.rb'

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'FTP Client'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = 1600

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 800

  # ========================================================================= #
  # === FTP_MAIN_ICON
  # ========================================================================= #
  FTP_MAIN_ICON = HOME_DIRECTORY_OF_USER_X+'DATA/images/PC/FTP/FTP.png'

  # ========================================================================= #
  # === SHALL_WE_DEBUG
  # ========================================================================= #
  SHALL_WE_DEBUG = false

  # ========================================================================= #
  # === MAIN_BG_COLOUR
  # ========================================================================= #
  MAIN_BG_COLOUR = :beige

  # ========================================================================= #
  # === MAIN_BG_COLOUR_PRELIGHT
  # ========================================================================= #
  MAIN_BG_COLOUR_PRELIGHT = :blanchedalmond

  # ========================================================================= #
  # === DEFAULT_USERNAME
  # ========================================================================= #
  DEFAULT_USERNAME = 'podi_278950456'

  # ========================================================================= #
  # === DEFAULT_REMOTE_HOST
  # ========================================================================= #
  DEFAULT_REMOTE_HOST = 'ftpupload.net'

  # ========================================================================= #
  # === DEFAULT_PASSWORD
  # ========================================================================= #
  DEFAULT_PASSWORD = '1aaaaaa'

  # ========================================================================= #
  # === DEFAULT_REMOTE_PORT
  # ========================================================================= #
  DEFAULT_REMOTE_PORT = 21

  # ========================================================================= #
  # === YES_THE_USER_IS_CONNECTED
  # ========================================================================= #
  YES_THE_USER_IS_CONNECTED = 'Yes!'

  # ========================================================================= #
  # === MY_TEMP
  # ========================================================================= #
  if ENV['MY_TEMP']
    MY_TEMP = ENV['MY_TEMP'].to_s+'/'
  elsif File.directory?('/home/Temp/')
    MY_TEMP = '/home/Temp/'
  else
    MY_TEMP = '/tmp/'
  end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    super(:vertical)
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    @title  = TITLE
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    @width  = WIDTH
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    @height = HEIGHT
    # ======================================================================= #
    # === @ftp_connection
    # ======================================================================= #
    @ftp_connection = nil
    # ======================================================================= #
    # === @debug    
    # ======================================================================= #
    @debug = SHALL_WE_DEBUG 
    # ======================================================================= #
    # === @hash_user_data
    # ======================================================================= #
    if is_on_roebe?
      # ===================================================================== #
      # This is valid for my home-system.
      # ===================================================================== #
      @hash_user_data = ::FtpParadise::RoebeFtpConstants.hash_ftp_dataset?['podserver']
    else
      # ===================================================================== #
      # Else we simply build up some default values; the regular user
      # has to overrule them anyway, via the GUI.
      # ===================================================================== #
      @hash_user_data = {}
      @hash_user_data['user_name'] = DEFAULT_USERNAME
      @hash_user_data['password']  = DEFAULT_PASSWORD
      @hash_user_data['url']       = DEFAULT_REMOTE_HOST
      @hash_user_data['port']      = DEFAULT_REMOTE_PORT
    end
    initialize_the_connection_image
    set_use_this_font(:dejavu_condensed_21)
    use_gtk_paradise_project_css_file
    infer_the_size_automatically
  end

  # ========================================================================= #
  # === initialize_the_connection_image
  # ========================================================================= #
  def initialize_the_connection_image
    # ======================================================================= #
    # === @connection_image
    # ======================================================================= #
    @connection_image = gtk_image(
      ::FtpParadise.project_base_dir?+'images/connection_image.png'
    )
    @connection_image.css_class('bblack1')
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    2
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === create_button_delete_remote_file
  # ========================================================================= #    
  def create_button_delete_remote_file
    @button_delete_remote_file = gtk_button('Delete remote file')
    @button_delete_remote_file.fancy_tooltips = 
      'Use this when you wish to remove a remote file.'
    @button_delete_remote_file.on_clicked {
      do_delete_remote_file
    }
  end

  # ========================================================================= #
  # === create_skeleton                                          (create tag)
  # ========================================================================= #
  def create_skeleton
    create_debug_button
    create_button_delete_remote_file
    create_status_icon
    create_statusbar
    @grid_user_data = gtk_grid
    @grid_user_data.set_column_spacing(5)
    @grid_user_data.set_row_spacing(5)
    @grid_user_data.set_size_request(300, 50)
    @grid_user_data.set_border_width(20)
    # ======================================================================= #
    # Populate the grid next.
    # ======================================================================= #
    @grid_user_data.set_left(
      text('Username:')
    )
    @entry_username = gtk_entry
    @entry_username.set_size_request(300, 50)
    @entry_username.set_text(@hash_user_data['user_name'])
    @entry_username.deselect
    @entry_username.css_class('pad8px')
    @grid_user_data.set_right(@entry_username)

    # ======================================================================= #
    # === The user Password
    # ======================================================================= #
    @grid_user_data.set_left(
      text('Password:')
    )
    @entry_user_password = gtk_entry
    @entry_user_password.set_size_request(300, 50)
    @entry_user_password.set_text(@hash_user_data['password'])
    @entry_user_password.deselect
    @entry_user_password.css_class('pad8px')
    @grid_user_data.set_right(@entry_user_password)

    # ======================================================================= #
    # === Remote host:
    # ======================================================================= #
    @grid_user_data.set_left(
      text('Remote host:')
    )
    @entry_remote_host = gtk_entry
    @entry_remote_host.set_size_request(300, 50)
    @entry_remote_host.set_text(@hash_user_data['url'])
    @entry_remote_host.css_class('pad8px')
    @grid_user_data.set_right(
      @entry_remote_host
    )
    @grid_user_data.set_left(
      text('Port:')
    )
    @entry_port = gtk_entry
    @entry_port.set_size_request(300, 50)
    @entry_port.set_text(@hash_user_data['port'].to_s)
    @entry_port.css_class('pad8px')
    @grid_user_data.set_right(
      @entry_port
    )

    @frame_for_the_grid_user_data = gtk_frame
    @frame_for_the_grid_user_data.set_text(
      ' User data for the FTP connection '
    )
    @frame_for_the_grid_user_data.add(@grid_user_data)
    @frame_for_the_grid_user_data.css_class('pad8px')
    @frame_for_the_grid_user_data.set_border_width(2)
    @frame_for_the_grid_user_data.set_size_request(400, 40)
  end

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    abort_on_exception
    hbox = gtk_hbox
    hbox.minimal(@frame_for_the_grid_user_data, 5)
    # ======================================================================= #
    # === widget2
    # ======================================================================= #
    vbox_is_the_user_connected = gtk_vbox
    vbox_is_the_user_connected.set_border_width(10)
    event_box_for_the_connection_image = gtk_event_box(@connection_image)
    event_box_for_the_connection_image.fancy_tooltips = 
      'Click on this image to connect to the specified, remote host.'
    event_box_for_the_connection_image.on_clicked {
      do_connect_to_the_remote_host_specified
    }
    event_box_for_the_connection_image.set_size_request(50, 50)
    vbox_is_the_user_connected.minimal(event_box_for_the_connection_image)
    @label_are_we_connected = text('No')
    vbox_is_the_user_connected.add(@label_are_we_connected)
    frame_is_the_user_connected = gtk_frame
    frame_is_the_user_connected.set_text(' Is the user connected: ')
    frame_is_the_user_connected.add(vbox_is_the_user_connected)

    # ======================================================================= #
    # Next add the box for the "Actions to perform".
    # ======================================================================= #
    frame_action_to_perform = gtk_frame
    frame_action_to_perform.set_text(' Action to perform: ')
    frame_action_to_perform.label_widget.fancy_tooltips =
      'Denote via this widget which action is to be performed.'
    vbox_action_to_perform = gtk_vbox
    vbox_action_to_perform.set_border_width(10)
    @button_upload = button('Upload')
    @button_upload.fancy_tooltips = 
      '<b>Upload</b> the selected local file to the remote host.'
    @button_upload.on_clicked {
      do_upload_the_selected_file
    }
    vbox_action_to_perform.add(@button_upload)
    @button_download = button('Download')
    @button_download.fancy_tooltips = 
      '<b>Download</b> the selected (<b>remote</b>) files/directories.'
    @button_download.on_clicked {
      do_download_the_remote_file_or_directory
    }
    vbox_action_to_perform.add(@button_download)
    vbox_action_to_perform.add(@button_delete_remote_file)
    vbox_action_to_perform.add(@button_debug)
    frame_action_to_perform.add(vbox_action_to_perform)

    draggable_widget_between_two_frames = draggable_up_and_down_pane(
      frame_is_the_user_connected,
      frame_action_to_perform
    )
    # hbox << frame_is_the_user_connected
    # hbox << frame_action_to_perform
    hbox.add(draggable_widget_between_two_frames)
    
    minimal(hbox, 2)
    hbox_with_the_two_file_listing_widgets = gtk_hbox
    vbox1 = gtk_vbox
    vbox1 << text('Local directory')
    @entry_local_directory = gtk_entry
    if File.directory? MY_TEMP
      @entry_local_directory.set_text(MY_TEMP)
    else
      @entry_local_directory.set_text(return_pwd)
    end
    @entry_local_directory.set_font(:hack_15)
    vbox1 << @entry_local_directory
    # ======================================================================= #
    # === @directory_content_widget1
    # ======================================================================= #
    @directory_content_widget1 = ::Gtk::DirectoryContentWidget.new {
      :do_not_use_widget_allowing_the_user_to_change_the_local_directory
    }
    @directory_content_widget1.set_start_dir(@entry_local_directory.text?)
    @directory_content_widget1.set_font(:hack_18)
    @entry_local_directory.on_enter {
      target = @entry_local_directory.text?
      if File.directory? target
        Dir.chdir(target)
      end
      @directory_content_widget1.set_use_this_directory(
        target
      )
      @directory_content_widget1.update
    }
    vbox1.maximal(@directory_content_widget1, 5)
    # Right right middle-vbox:
    vbox2 = gtk_vbox
    vbox2 << text('Remote directory')
    @directory_content_widget2 = ::Gtk::DirectoryContentWidget.new {
      :do_not_use_widget_allowing_the_user_to_change_the_local_directory
    }
    # ======================================================================= #
    # === @directory_content_widget2
    # ======================================================================= #
    @directory_content_widget2.set_font(:hack_18)
    @directory_content_widget2.clear # This widget is empty on startup.
    @entry_remote_directory = gtk_entry
    @entry_remote_directory.set_font(:hack_15)
    vbox2 << @entry_remote_directory
    vbox2 << @directory_content_widget2
    # ======================================================================= #
    # Next add vbox1 and vbox2 together.
    # ======================================================================= #
    up_and_down_spacer = up_and_down_pane(vbox1, vbox2)
    up_and_down_spacer.position = 780
    hbox_with_the_two_file_listing_widgets.maximal(up_and_down_spacer)
    minimal(hbox_with_the_two_file_listing_widgets, 2)
    do_style_all_buttons_in_a_uniform_manner
  end

  # ========================================================================= #
  # === user_password?
  # ========================================================================= #
  def user_password?
    @entry_user_password.text?
  end

  # ========================================================================= #
  # === remote_url?
  # ========================================================================= #
  def remote_url?
    @entry_remote_host.text?
  end

  # ========================================================================= #
  # === set_remove_directory
  # ========================================================================= #
  def set_remote_directory(i = '/')
    @entry_remote_directory.set_text(i.to_s)
  end

  # ========================================================================= #
  # === GtkParadise::GUI::Gtk::FtpParadiseModule.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::FtpParadise::GUI::Gtk::FtpParadise.new(i)
    r = ::Gtk.run
    r << _
    r.set_size_request(_.width?, _.height?)
    r.automatic_title
    r.top_left_then_run
  end

  # ========================================================================= #
  # === clear_right_listing
  # ========================================================================= #
  def clear_right_listing
    @directory_content_widget2.clear
  end

  # ========================================================================= #
  # === are_we_connected?
  # ========================================================================= #
  def are_we_connected?
    @label_are_we_connected.text? == YES_THE_USER_IS_CONNECTED
  end

  # ========================================================================= #
  # === do_download_the_remote_file_or_directory
  # ========================================================================= #
  def do_download_the_remote_file_or_directory(
      this_file = @directory_content_widget2.selected?
    )
    if are_we_connected?
      # ===================================================================== #
      # In this case we can proceed to download the remote file. We have
      # to make sure the remote file exists, though.
      # ===================================================================== #
      begin
        ::FtpParadise.download(
          this_file,
          @ftp_connection
        )
      rescue Exception => error
        e 'The file '+this_file.to_s+' does not exist, though.'
        pp error.class
        pp error
      end
    else
      e 'But there is no active FTP connection right now.'
    end
  end

  # ========================================================================= #
  # === is_on_roebe?
  # ========================================================================= #
  def is_on_roebe?
    ::FtpParadise.is_on_roebe?
  end

  # ========================================================================= #
  # === do_connect_to_the_remote_host_specified
  #
  # This is the method that will connect to the remove host specified.
  # ========================================================================= #
  def do_connect_to_the_remote_host_specified(
      remote_url = remote_url?
    )
    @ftp_connection = Net::FTP.open(remote_url)
    # @ftp_connection.passive = true
    user_name     = user_name?
    user_password = user_password?
    @ftp_connection.login(user_name, user_password)
    entries = @ftp_connection.list('*')
    @label_are_we_connected.set_text(YES_THE_USER_IS_CONNECTED)
    clear_right_listing
    set_remote_directory('/')
    modified_entries = entries.map {|line|
      if line.empty? or line.end_with?('.')
        line = nil
      end
      if line and line.end_with? ':'
        line = line.dup
        line.chop!
        line << '/'
      end
      line
    }.compact
    modified_entries = modified_entries.map! {|line|
      splitted = line.split(' ')
      filename = splitted.last
      type = 'dir or file here' # Default.
      if line.end_with? '/'
        type = 'directory'
      end
      [filename, type,'LAST_MODIFIED_GOES_IN_HERE']
    }
    @directory_content_widget2.feed_this_data(modified_entries)
  end; alias do_connect do_connect_to_the_remote_host_specified # === do_connect

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    @debug
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_skeleton_then_connect_skeleton
    do_connect if is_on_roebe? # Connect at once on my home system.
  end

  # ========================================================================= #
  # === do_style_all_buttons_in_a_uniform_manner
  # ========================================================================= #
  def do_style_all_buttons_in_a_uniform_manner
    return_all_buttons.each {|this_button|
      this_button.modify_background(:active,   MAIN_BG_COLOUR )
      this_button.modify_background(:normal,   MAIN_BG_COLOUR )
      this_button.modify_background(:prelight, MAIN_BG_COLOUR_PRELIGHT)
      this_button.set_border_width(1)
      this_button.bblack1
    }
  end

  # ========================================================================= #
  # === add_status_message
  #
  # Use this to append to the Statusbar.
  # ========================================================================= #
  def add_status_message(i)
    @status_bar.push(@status_bar_context_id, i)
  end

  # ========================================================================= #
  # === create_status_icon
  # ========================================================================= #
  def create_status_icon
    @status_icon = gtk_status_icon
    if File.exist? FTP_MAIN_ICON
      @status_icon.pixbuf = GdkPixbuf::Pixbuf.new(file: FTP_MAIN_ICON)
    end
    @status_icon.fancy_tooltips = 'StatusIcon'
  end

  # ========================================================================= #
  # === create_statusbar
  #
  # Create the Statusbar.
  # ========================================================================= #
  def create_statusbar
    @status_bar = gtk_statusbar
    if @status_bar.respond_to?(:has_resize_grip=)
      @status_bar.has_resize_grip = true
    end
    @status_bar_context_id = @status_bar.get_context_id('Gtk Ftp Client') 
  end

  # ========================================================================= #
  # === create_debug_button                         (debug tag, debug button)
  # ========================================================================= #
  def create_debug_button
    @button_debug = gtk_button('_Debug') # The debug-button.
    @button_debug.on_clicked {
      e 'Debugging FTP Client.'
      ecomment ('The password is: ').ljust(20)+password?
      ecomment 'The login name is: '+username?, token: ':'
    }
  end

  # ========================================================================= #
  # === user_name?
  # ========================================================================= #
  def user_name?
    @entry_username.text?
  end; alias username? user_name? # === username?

  # ========================================================================= #
  # === do_delete_remote_file
  #
  # Wrapper to delete a remote file.
  # ========================================================================= #
  def do_delete_remote_file
    ewarn 'Deleting this remote file:'
    efancy selection?(:right)
    @ftp_connection.remove_file( selection?(:right) )
    update_everything
  end

  # ========================================================================= #
  # === output_coloured_line
  #
  # Makes a nicely coloured line.
  # ========================================================================= #
  def output_coloured_line(
      input, optional_colour = 'yelb'
    )
    cliner {
      e sfancy(input)
    }
  end

  # ========================================================================= #
  # === return_default_user_name_based_on_selected_host
  # ========================================================================= #
  def return_default_user_name_based_on_selected_host(
      selected_host = host?
    )
    selected_host = FtpParadise::RoebeFtpConstants.
                    return_user_name_based_on_given_host(selected_host)
    return selected_host
  end

  # ========================================================================= #
  # === connect_to_remote_host
  #
  # Use this method when connecting to the remote host.
  # ========================================================================= #
  def connect_to_remote_host(this_host = host?)
    this_host = this_host.to_s
    @ftp_connection.connect_to(this_host, :be_verbose)
    add_status_message 'Connecting to '+this_host.to_s+' now.'
    try_to_sync_label_remote_dir
  end

  # ========================================================================= #
  # === do_upload_the_selected_file
  # ========================================================================= #
  def do_upload_the_selected_file
    _ = left_tree_view?.selection?
    if _
       ::FtpParadise.upload(_, :short_variant, @ftp_connection)
    else
      e 'Nothing was selected.'
    end
  end

  # ========================================================================= #
  # === left_tree_view?
  # ========================================================================= #
  def left_tree_view?
    @directory_content_widget1
  end

end; end; end

# =========================================================================== #
# === FtpParadise.do_start_gui
#
# This will output what will be done, then start the GUI bindings.
#
# This method can be invoked via:
#
#   ftpparadise --gui
#
# =========================================================================== #
def self.do_start_gui
  opnn; puts 'We will next start the GUI.'
  FtpParadise::GUI::Gtk::FtpParadiseModule.start_gtk
end

# =========================================================================== #
# === FtpParadise.start_gtk
# =========================================================================== #
def self.start_gtk
  FtpParadise::GUI::Gtk::FtpParadiseModule.run
end

# =========================================================================== #
# === FtpParadise.gtk_widget
#
# This method will return a useable GTK widget that can be embedded into
# other applications.
# =========================================================================== #
def self.gtk_widget
  FtpParadise::GUI::Gtk::FtpParadiseModule
end

end