#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/menu.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp

  require 'ftp_paradise/interactive_ftp/upload.rb'

  begin
    require 'verbose_truth'
  rescue LoadError; end

  begin
    require 'rcfiles/requires/require_the_expand_cd_aliases.rb'
  rescue LoadError; end

  require 'ftp_paradise/toplevel_methods/ftp_object.rb'

  # ========================================================================= #
  # === vte
  # ========================================================================= #
  def vt(i)
    VerboseTruth[i].to_s
  end

  # ========================================================================= #
  # === esystem
  # ========================================================================= #
  def esystem(i)
    e i
    system i
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # Do note that @user_input will always be an Array.
  # ========================================================================= #
  def menu(
      i = @user_input
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      # ===================================================================== #
      # Handle arguments first. We have to check if our input contains
      # any ' ' characters or not.
      # ===================================================================== #
      if i.include? ' '
        @splitted = i.split(' ')
        @first_argument       = @splitted[1]
        @s = @second_argument = @splitted[2]
        @all_arguments        = @splitted[1..-1] # This here does not contain the first argument.
        i                     = @splitted[0]
      else
        # =================================================================== #
        # Reset again here so that we have a clean initial state.
        # =================================================================== #
        reset_instance_variables_that_hold_all_arguments
      end
      f = @first_argument
      case i # case tag
      # ===================================================================== #
      # === toshevy
      #
      # This is my "convenience" login.
      # ===================================================================== #
      when /^-?-?toshevy$/,
           /^-?-?shevy$/i,
           'shev', # Login as shevy.
           'iftp',
           'remote'
        do_login_to_shevy_and_consider_showing_the_remote_files
      # ===================================================================== #
      # === uniwien_homepage
      # ===================================================================== #
      when /^-?-?uniwien(_|-)?homepage$/
        do_login(:uniwien_homepage)
        if SHOW_REMOTE_FILES_UPON_SUCCESSFUL_LOGIN
          show_remote_files :also_show_filesize # Also display this since as of March 2015.
        end
      # ===================================================================== #
      # === login
      # ===================================================================== #
      when 'login',
           'log',
           'cto',
           'con',
           'connect_to',
           'do_login',
           'conn',
           'connectto',
           'connect2',
           /^-?-?do(_|-)?connect$/i,
           'connect' # ← easier alias for now.
        do_login(f) { :be_verbose }
      # ===================================================================== #
      # === set_url
      # ===================================================================== #
      when /^-?-?set(_|-)?url$/i
        e 'Setting to the remote host called '+sfancy(f)+'.'
        set_url(f)
      # ===================================================================== #
      # === set_password
      # ===================================================================== #
      when 'set_password',
           'setpassword',
           'password'
        set_password(f)
      # ===================================================================== #
      # === setname
      #
      # This entry point allows us to set the name of the user.
      # ===================================================================== #
      when 'setname',
           'set_name',
           'set_user',
           'setuser',
           'set_user_name',
           'username'
        set_name(f)
      # ===================================================================== #
      # === everything
      #
      # Can also be invoked via show_host_user_name_port_and_password.
      # ===================================================================== #
      when 'everything',
           /show_?host_?user_?name_?port_?and_?password/,
           'username?',
           'uname?',
           'user?'
        show_host_user_name_port_and_password
      # ===================================================================== #
      # === chmod
      # ===================================================================== #
      when 'chmod'
        filename = 'htop-2.2.0.tar.xz'
        try_to_chmod_this_remote_file(filename)
      # ===================================================================== #
      # === remote_host?
      # ===================================================================== #
      when /^remote(_|-)host\??/
        e sfancy(FtpParadise.remote_host?)
      # ===================================================================== #
      # === vim
      # ===================================================================== #
      when /^vim/
        esystem "vim #{f}"
      # ===================================================================== #
      # === set_editor
      #
      # This entry point allows us to specify a different editor.
      # ===================================================================== #
      when /^set(-|_| )?editor$/ # === set_editor vim
        set_editor(f)
      # ===================================================================== #
      # === editor?
      # ===================================================================== #
      when 'editor?',
           /^report(-|_| )?which(-|_| )?editor(-|_| )?is(-|_| )?in(-|_| )?use$/i
        report_which_editor_is_in_use
      # ===================================================================== #
      # === edit
      # ===================================================================== #
      when 'edit',
           'bluefish',
           'bl',
           'open',
           /^open(-|_| )?this(-|_| )?file(-|_| )?in(-|_| )?editor$/,
           'open?'
        open_this_file_in_editor(f)
      # ===================================================================== #
      # === binary_upload
      # ===================================================================== #
      when /^binary(-|_| )?upload$/,
           /^upload(-|_| )?binary$/,
           'bupload',
        do_use_binary_mode
        upload(f)
      # ===================================================================== #
      # === upload_this_file
      #
      # This entry point allows us to upload a local file to a remote
      # host.
      #
      # Invocation example:
      #
      #   upload /Depot/j/test.md
      #
      # ===================================================================== #
      when /^-?-?upload(-|_| )?this(-|_| )?file$/,
           'upload',
           'upoad',
           'upl',
           'up',
           'icopy', # copy all from local to remote dir
           'uload'
        upload(@all_arguments) # bl $FTP_PARADISE/
      # ===================================================================== #
      # === iftp download
      #
      # This entry point allows us to download a remote file, via
      # ruby's FTP interface.
      # ===================================================================== #
      when /^-?-?download$/,'get','down','do','dow','dload','dl',
           'wget','wg'
        download_remote_file(f)
      # ===================================================================== #
      # === remote_dirs?
      # ===================================================================== #
      when 'remote_dirs?','dirs?','dir?','dir','dirs','onlydirs',
           'remotedirs?',
           /^-?-?show(-|_| )?remote(-|_| )?directories$/
        show_remote_directories
      # ===================================================================== #
      # === rpwd
      # ===================================================================== #
      when 'rpwd',
           /^remote(-|_| )?pwd/,
          'remote_pwd?','remote_path?','remote_dir?','rdir',
           'lpwd','rpwd?','rcd?'
        report_remote_directory
      # ===================================================================== #
      # === upload_directory
      # ===================================================================== #
      when /^upload(_|-)?directory$/
        upload_directory(f)
      # ===================================================================== #
      # === raw_list
      #
      # This entry point will just do .list() on the ftp-object that
      # holds our remote connection.
      # ===================================================================== #
      when /^raw(_|-)?list$/
        pp ftp?.raw_list
      # ===================================================================== #
      # === raw
      # ===================================================================== #
      when 'raw','@array_file_listing','array_file_listing',
           'l1','list_content','raw?'
        pp ftp?.array_file_listing
      # ===================================================================== #
      # === remote_remove
      # ===================================================================== #
      when 'remote_remove','rrmdir','remove_remote_directory','removeremove',
           'rremove','rrmfile','rrm','remove_remote','remote_rm','remoterm',
           'remove_remote_file',
           'rrem',
           'removeremotedirectory',
           'del',
           'remoteremove',
           'removeremote',
           'remove_remove',
           'remoteremote',
           'remove_remote_dir',
           'rmdir',
           'remotermdir',
           'deldir'
        _ = a?
        if user_input?.first.include?(' ') and
          _.last.end_with?('.pdf','.pdf"') # <-- Yup, hackish. For now we only make an exception for .pdf files.
          _ = _.join(' ')
        end
        remote_remove(_)
      # ===================================================================== #
      # === nlst
      # ===================================================================== #
      when 'nlst','nls','nlst?'
        e ftp?.nlst
      # ===================================================================== #
      # === list?
      # ===================================================================== #
      when 'list?','list'
        pp ftp?.list?
      # ===================================================================== #
      # === q
      # ===================================================================== #
      when *ARRAY_VALID_OPTIONS_FOR_EXITING_THE_SHELL, :break
        :break # quit tag
      # ===================================================================== #
      # === @_
      # ===================================================================== #
      when '@_','@_?','__','ftp?','ftp_object?','ftpobject?'
        pp ftp?
      # ===================================================================== #
      # === FtpParadise.dataset?
      # ===================================================================== #
      when 'FtpParadise.dataset?','dataset?'
        pp FtpParadise.dataset?
      # ===================================================================== #
      # === ll
      # ===================================================================== #
      when 'll',
           'llpwd',
           'showlocalfiles',
           'show_local_files',
           'ls',
           'local','l'
        show_local_entries
      # ===================================================================== #
      # === remote_ls
      #
      # This entry point will show which remote files exist.
      # ===================================================================== #
      when 'remote_ls',
           'remotell',
           'listing',
           'lll',
           'lls',
           'llll',
           'remotecontent?',
           '3l','show_remote_files','l?','listing?',
           'rls',
           '3list',
           'rll',
           'remotels',
           /remote_?listing/,
           'showremotecontent','showremote'
        e "The remote file listing at #{sfancy(remote_host?)} is:"
        show_remote_files
      # ===================================================================== #
      # === rcd
      #
      # This entry point allows us to change to a remote directory.
      #
      # Invocation example:
      #
      #   rcd htdocs
      #
      # ===================================================================== #
      when 'rcd',
           /^-?-?remote(-|_| )?change(-|_| )?directory$/,
           'rchange_dir','lcd','remotecd','rc','remote_cd'
        remote_change_directory(f, :show_remote_file_listing)
      # ===================================================================== #
      # === anonymous
      # ===================================================================== #
      when 'anonymous','anon'
        enable_anonymous_login
      # ===================================================================== #
      # === port?
      # ===================================================================== #
      when 'port?', /report_?port/
        report_port
      # ===================================================================== #
      # === mode?
      # ===================================================================== #
      when 'mode?','mode','transfermode?'
        e 'The mode for InteractiveFtp is: '+sfancy(@mode.to_s)
      # ===================================================================== #
      # === show_user_and_password
      # ===================================================================== #
      when /show_?user_?and_?password/
        show_user_and_password
      # ===================================================================== #
      # === show_hosts
      # ===================================================================== #
      when /^-?-?show_?hosts/,'hosts?'
        show_hosts
      # ===================================================================== #
      # === move_remote_file
      # ===================================================================== #
      when /move_?remote_?file/,'rmv','mmv','remote_mv'
        move_remote_file(a)
      # ===================================================================== #
      # === download_binary_file
      # ===================================================================== #
      when 'download_binary_file','downloadbinaryfile',
           'getbinaryfile'
        ftp?.download_binary_file(f)
      # ===================================================================== #
      # === file?
      # ===================================================================== #
      when 'file?','upload?','f?'
        consider_reporting_file
      # ===================================================================== #
      # === www
      # ===================================================================== #
      when 'www','sinatra','show_www_component','www?'
        show_www_component
      # ===================================================================== #
      # === binary?
      # ===================================================================== #
      when 'binary?'
        e vt(@mode == :binary)
      # ===================================================================== #
      # === is_directory?
      # ===================================================================== #
      when 'is_directory?','isdirectory?','isdir?','is_a_directory?'
        e is_dir(f).to_s
      # ===================================================================== #
      # === active
      # ===================================================================== #
      when 'active'
        set_active
      # ===================================================================== #
      # === passive
      # ===================================================================== #
      when 'passive'
        set_passive
      # ===================================================================== #
      # === download?
      # ===================================================================== #
      when 'download?'
        download?
      # ===================================================================== #
      # === stat
      # ===================================================================== #
      when 'stat','stat?'
        stat_remote_file(f)
      # ===================================================================== #
      # === a1?
      # ===================================================================== #
      when 'a1?' # Feedback the A1 values.
        pp A1
      # ===================================================================== #
      # === ..
      # ===================================================================== #
      when '..'
        change_directory(i) # Special cd instruction.
      # ===================================================================== #
      # === ascii
      # ===================================================================== #
      when 'ascii'
        use_ascii_mode(true)
      # ===================================================================== #
      # === disable
      # ===================================================================== #
      when 'disable'
        disable f
      # ===================================================================== #
      # === info
      # ===================================================================== #
      when 'info','debug','deb','debug_it','debugit'
        debug_it
      # ===================================================================== #
      # === show_full_names
      # ===================================================================== #
      when 'show_full_names',
           'not_show',
           'notshow',
           'show',
           'show_names'
        toggle_show_full_names
      # ===================================================================== #
      # === base_dir?
      # ===================================================================== #
      when 'base_dir?','basedir?'
        e PROJECT_BASE_DIRECTORY
      # ===================================================================== #
      # === config?
      # ===================================================================== #
      when 'config?','configuration?','ccc','config'
        display_configuration_options
        pp @config
      # ===================================================================== #
      # === local_files
      # ===================================================================== #
      when 'local_files','localfiles'
        show_local_files(true) # true for dirs-only.
      # ===================================================================== #
      # === set
      # ===================================================================== #
      when 'set','set_file','setfile','file','assign'
        if @first_argument.to_s =~ /^\d+/ # upload only a specific number.
          @first_argument = Dir['*'][@first_argument.to_i + 1]
        end
        set_file(f)
        e "Setting @file to `#{sfile(file?)}` now." if be_verbose
      # ===================================================================== #
      # === available?
      # ===================================================================== #
      when 'available?','show_available_connections'
        show_available_connections
      # ===================================================================== #
      # === toggle2
      # ===================================================================== #
      when 'toggle2','toggle_modes','togglemodes','t2' # toggle the mode.
        toggle_modes
      # ===================================================================== #
      # === last
      # ===================================================================== #
      when 'last'
        last_action = @history.last
        check_against_menu(last_action)
      # ===================================================================== #
      # === localdir?
      # ===================================================================== #
      when 'localdir?','ldir'
        report_local_dir
      # ===================================================================== #
      # === binary
      # ===================================================================== #
      when 'binary'
        use_binary_mode(true)
      # ===================================================================== #
      # === yaml?
      # ===================================================================== #
      when 'yaml?'
        show_ftp_yaml_file
      # ===================================================================== #
      # === remove_local
      # ===================================================================== #
      when 'remove_local','delete','rm','remove','rem'
        remove_local(f)
      # ===================================================================== #
      # === site?
      # ===================================================================== #
      when 'site?'
        e ftp?.site
      # ===================================================================== #
      # === mdtm
      # ===================================================================== #
      when 'mdtm','modtime?','modtime','modification_time_of?',
           'show_modtime_of','showmodtimeof','mtime?' # mtime? is probably only temporary.
        show_modtime_of(f)
      # ===================================================================== #
      # === downloaded?
      # ===================================================================== #
      when 'downloaded?'
        show_which_files_we_did_download_so_far
      # ===================================================================== #
      # === set_login
      # ===================================================================== #
      when /set_?login/,'login='
        set_login(a?)
      # ===================================================================== #
      # === cd
      # ===================================================================== #
      when 'cd','change_dir'
        change_directory(f)
      # ===================================================================== #
      # === status?
      # ===================================================================== #
      when 'status?','info?','stats?','s?','status'
        show_status
      # ===================================================================== #
      # === connected_to?
      # ===================================================================== #
      when 'connected?','c?','remote?','host?',
           'connected_to?','url?','cto?','?'
        connected_to?
      # ===================================================================== #
      # === remotemkdir
      # ===================================================================== #
      when 'rmkdir','remote_mkdir','remotemkdir','mkdir_remote'
        create_remote_directory(f)
      # ===================================================================== #
      # === is_file?
      # ===================================================================== #
      when /is_?file\??/
        f = convert_number_to_file(f)
        e ftp?.is_file?(f).to_s
     # ===================================================================== #
      # === change
      # ===================================================================== #
      when 'change'
        change @first_argument
      # ===================================================================== #
      # === full
      # ===================================================================== #
      when 'full'
        e 'We will show the full path.'
        do_show_full_names
      # ===================================================================== #
      # === full_names?
      # ===================================================================== #
      when 'full_names?','full?','names?','name?','fullnames?',
           'full_path?','fullpath?'
        will_we_show_full_names
      # ===================================================================== #
      # === rename
      # ===================================================================== #
      when 'rename','remote_rename','remoterename','remotemv'
        rename(f?, @s)
      # ===================================================================== #
      # === remote_remove
      # ===================================================================== #
      when 'mv','move_file','movefile','move_lovel_file'
        move_local_file(f, @s)
      # ===================================================================== #
      # === last?
      # ===================================================================== #
      when 'last?'
        e @history.last
      # ===================================================================== #
      # === history
      # ===================================================================== #
      when 'history','history?'
        show_history
      # ===================================================================== #
      # === cat
      # ===================================================================== #
      when 'cat'
        display_file(f)
      # ===================================================================== #
      # === touch
      # ===================================================================== #
      when 'touch'
        create_file(f)
      # ===================================================================== #
      # === mkdir
      # ===================================================================== #
      when 'mkdir'
        create_directory(f, :be_verbose)
      # ===================================================================== #
      # == set_port
      # ===================================================================== #
      when 'set_port','setport','port'
        set_port(f) # setport 33791
      # ===================================================================== #
      # === debug?
      # ===================================================================== #
      when 'debug?'
        e verbose_truth(@config.debug.to_s)
        ftp?.status?
      # ===================================================================== #
      # === help?
      # ===================================================================== #
      when *ARRAY_HELP_OPTIONS
        show_help
      # ===================================================================== #
      # === dated?
      # ===================================================================== #
      when 'dated?','dated'
        show_date_sorted_entries
      # ===================================================================== #
      # === date?
      # ===================================================================== #
      when 'date?','date'
        e return_time+' '+return_date
      # ===================================================================== #
      # === welcome
      # ===================================================================== #
      when 'welcome','show_welcome_message','welcome?','welc'
        show_ftp_server_welcome
      # ===================================================================== #
      # === show_ftp_docu
      # ===================================================================== #
      when 'show_ftp_docu','ftp_docu?','docu?','ftp_docu'
        show_ftp_docu
      # ===================================================================== #
      # === enable_readline
      # ===================================================================== #
      when /^-?-?enable(_|-)?readline$/
        enable_readline
      # ===================================================================== #
      # === disconnect
      # ===================================================================== #
      when 'disconnect','dconnect','out'
        disconnect
      # ===================================================================== #
      # === toggle
      # ===================================================================== #
      when 'toggle'
        toggle(f)
      # ===================================================================== #
      # === colours?
      # ===================================================================== #
      when 'colours?','show_colours'
        e 'Will we use colours: '+vt(use_colours?)        
        show_colours
      # ===================================================================== #
      # === nofull
      # ===================================================================== #
      when 'nofull'
        e 'We will not show the full path.'
        do_not_show_full_names
      # ===================================================================== #
      # === last_response_code?
      # ===================================================================== #
      when 'last_response_code?','last_response_code','lastresponsecode',
           'lastresponsecode?','report_last_response_code'
        report_last_response_code
      # ===================================================================== #
      # === last_response?
      # ===================================================================== #
      when 'last_response?','last_response','lastresponse','lastresponse?',
           'lastconnected','laststatus'
        report_last_response
      # ===================================================================== #
      # === noop
      # ===================================================================== #
      when 'noop'
        ftp?.noop
        e ftp?.last_response
      # ===================================================================== #
      # === pwd
      # ===================================================================== #
      when 'pwd','pwd?','pw'
        report_local_directory
      # ===================================================================== #
      # === readline?
      # ===================================================================== #
      when 'readline?'
        e verbose_truth(use_readline?)
      # ===================================================================== #
      # === iftp --no-readline
      # ===================================================================== #
      when /^-?-?no(-|_)?readline$/,
           /^-?-?do(-|_| )?not(-|_| )?use(-|_| )?readline$/
        do_not_use_readline
      # ===================================================================== #
      # === datafile?
      # ===================================================================== #
      when 'datafile?'
        e datafile?
      # ===================================================================== #
      # === delete
      #
      # Invocation example:
      #   delete files for your website should be uploaded here!
      # ===================================================================== #
      when /^-?-?delete$/,
           /^-?-?del$/
        remote_remove_file(@all_arguments.join(' ').strip)
      # ===================================================================== #
      # === remote_remove_file
      #
      # This entry point allows us to remove a (or several) remote file(s).
      #
      # Usage example:
      #   rcd htdocs; remote-remove index2.html
      # ===================================================================== #
      when /^-?-?remote(-|_| )?remove(-|_| )?files?$/,
           /^-?-?remote(-|_| )?remove$/
        remote_remove_file(a?)
      # ===================================================================== #
      # === remote-create-directory
      # ===================================================================== #
      when /^-?-?remote(-|_| )?create(-|_| )?directory$/,
           /^-?-?remote(-|_| )?mkdir$/
        remote_create_directory(a?)
      # ===================================================================== #
      # === quiet
      # ===================================================================== #
      when 'quiet','be_quiet'
        ConvertGlobalEnv.be_quiet # So that it will be silent.
      # ===================================================================== #
      # === password?
      # ===================================================================== #
      when 'password?',
           'pw?'
        e password?
      # ===================================================================== #
      # === help
      # ===================================================================== #
      when /help/
        show_help
      # ===================================================================== #
      # === list2
      # ===================================================================== #
      when 'list2','rlist' # Debug here.
        ftp?.list
      # ===================================================================== #
      # === time
      # ===================================================================== #
      when 'time','time?'
        e return_time
      # ===================================================================== #
      # === version?
      # ===================================================================== #
      when 'version?',/^report(_|-)?version/
        report_version
      else # else tag.
        _ = Rcfiles::ExpandCdAliases.new(i) # To expand our cd-aliases.
        if _._ # If we have found it.
          change_directory(_._) # My cd-aliases go in here.
        elsif i =~ /^\d+$/
          remote_cd(i) # perform a remote cd here.
        elsif i[0,1] == '?' # If has leading '?'
          stat_remote_file(i[1..-1])
        else
          display_prompt # Else display the prompt by default.
        end
      end
      reset_instance_variables_that_hold_all_arguments
      return @run_mode
    end
  end; alias check_against_menu menu # === check_against_menu

end; end