#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/misc.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp

  require 'ftp_paradise/interactive_ftp/directory_handling.rb'

  require 'ftp_paradise/toplevel_methods/determine_user_dataset_from_this_hash.rb'

  # ========================================================================= #
  # === set_editor
  # ========================================================================= #
  def set_editor(
      i, be_verbose = true
    )
    i = i.first if i.is_a? Array
    if be_verbose
      opnn; e "Now using the editor called `#{sfancy(i)}`."
    end
    @use_this_editor = i
  end

  # ========================================================================= #
  # === report_which_editor_is_in_use
  # ========================================================================= #
  def report_which_editor_is_in_use
    opnn; e "The editor in use is `#{sfancy(@use_this_editor)}`."
  end

  # ========================================================================= #
  # === open_this_file_in_editor           (edit tag, editor tag, open tag)
  #
  # If you pass as first argument to this method the string 'ALL', we
  # will try to open the (local) project files in your editor.
  #
  # To test this, do:
  #
  #   open ALL
  #   open YAML
  #
  # ========================================================================= #
  def open_this_file_in_editor(
      this_file = @file
    )
    this_file = ARRAY_PROJECT_FILES if this_file.nil?
    if this_file.is_a? Array
      this_file.each {|entry| open_this_file_in_editor(entry) }
    else
      this_file = this_file.to_s
      if this_file == 'TODO'
        this_file = ARRAY_PROJECT_FILES.select {|entry| entry.include? 'TODO' }[0]
      end
      if this_file.to_s =~ /^\d+$/ # If only numbers were given.
        this_file = Dir['*'].sort[this_file.to_i - 1]
      end
      this_file = NAME_OF_THIS_FILE if this_file.empty?
      if this_file == 'ALL' and ! File.exist?('ALL')
        open_this_file_in_editor(ARRAY_PROJECT_FILES)
      elsif this_file == 'YAML' and ! File.exist?('YAML')
        open_this_file_in_editor(YAML_FILE)
      else
        _ = @use_this_editor+" #{this_file}"
        if File.exist?(this_file) && File.file?(this_file)
          unless has_unwanted_file_suffix?(this_file)
            e 'Loading '+sfile(this_file)+' in editor '+
              'now as '+simp('USE_THIS_EDITOR')+' was set '+
              'to true:'
            e '  '+_+' &'
            `#{_}`
          end
          set_file(this_file) # Since 24.12.2011 we also assign it.
        else
          e swarn('But the file `')+sfile(this_file)+
            swarn('` does not exist.')
        end
      end
    end
  end; alias open_in_editor open_this_file_in_editor # === open_in_editor

  # ========================================================================= #
  # === determine_user_dataset_from_this_hash
  #
  # This method can set relevant entries from an input Hash.
  # ========================================================================= #
  def determine_user_dataset_from_this_hash(
      i,
      optional_use_this_entry = :default
    )
    if optional_use_this_entry
      case optional_use_this_entry
      # ===================================================================== #
      # === :default
      # ===================================================================== #
      when :default
        optional_use_this_entry = :bplaced
      end
    end
    FtpParadise.determine_user_dataset_from_this_hash(i)
  end

  # ========================================================================= #
  # === set_password
  # ========================================================================= #
  def set_password(
      i, be_verbose = be_verbose?
    )
    i = i.to_s
    if be_verbose
      e "Setting password to #{sfancy(i)} now."
    end
    FtpParadise.set_password(i)
  end

  # ========================================================================= #
  # === pass_commandline_arguments_into_the_menu
  # ========================================================================= #
  def pass_commandline_arguments_into_the_menu
    menu(@commandline_arguments)
  end

  # ========================================================================= #
  # === set_port
  #
  # Use this method to set the port.
  # ========================================================================= #
  def set_port(
      i = 21, be_verbose = false
    )
    if be_verbose
      e "Setting port to #{sfancy(i)} now."
    end
    ftp?.set_port(i)
  end

  # ========================================================================= #
  # === remote_url?
  # ========================================================================= #
  def remote_url?
    FtpParadise.remote_url?
  end; alias remote_host? remote_url? # === remote_host?

  # ========================================================================= #
  # === remote_remove_file
  # ========================================================================= #
  def remote_remove_file(
      i = @first_argument
    )
    if i.is_a? Array
      i.each {|entry| remote_remove_file(entry) }
    else
      e "Trying to remove `#{sfile(i)}` next:"
      @ftp.delete(i)
    end
  end; alias remote_remove_files remote_remove_file # === remote_remove_files

  # ========================================================================= #
  # === display_file
  #
  # Use this method to display a file in question.
  # ========================================================================= #
  def display_file(i)
    if File.exist? i
      _ = File.readlines(i).join
      e _
    else
      ewarn "File #{sfile(i)} does not exist."
    end
  end

  # ========================================================================= #
  # === report_version
  # ========================================================================= #
  def report_version
    e sfancy(FtpParadise::VERSION)
  end

  # ========================================================================= #
  # === all_arguments?
  #
  # The alias a? may be an easier wrapper over @all_arguments.
  # ========================================================================= #
  def all_arguments?
    @all_arguments
  end; alias a? all_arguments? # === a?

  # ========================================================================= #
  # === try_to_chmod_this_remote_file
  # ========================================================================= #
  def try_to_chmod_this_remote_file(
      filename, chmod_value_to_use = '0777'
    )
    e "Trying to chmod the remote file #{filename} next."
    ftp?.sendcmd("SITE CHMOD #{chmod_value_to_use} #{filename}")
  end

  # ========================================================================= #
  # === password?
  # ========================================================================= #
  def password?
    FtpParadise.password?
  end

end; end