#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/mode.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp # === FtpParadise::InteractiveFtp

  # ========================================================================= #
  # === use_binary_mode
  #
  # This enables use the binary mode.
  # ========================================================================= #
  def use_binary_mode(
      be_verbose = false
    )
    case be_verbose
    when :be_quiet
      be_verbose = false
    when :be_verbose
      be_verbose = true
    end
    e 'Using binary mode now.' if be_verbose
    set_mode :binary
  end; alias set_binary use_binary_mode # === set_binary

  # ========================================================================= #
  # === set_mode
  #
  # Set the mode here. We default to :ascii mode.
  #
  # There are only two valid modes:
  #
  #   :ascii
  #   :binary
  #
  # ========================================================================= #
  def set_mode(
      i = :ascii
    )
    @mode = i
    ftp?.set_transfer_mode(
      @mode, :be_quiet # Cascade the variable down into ftp_lib.rb.
    )
  end

  # ========================================================================= #
  # === do_use_binary_mode
  # ========================================================================= #
  def do_use_binary_mode
    use_binary_mode(:be_quiet)
  end

  # ========================================================================= #
  # === change
  #
  # This method can be used to change the main transfer mode.
  # ========================================================================= #
  def change(i)
    case i
    # ======================================================================= #
    # === mode
    # ======================================================================= #
    when 'mode' # Change the mode that we use.
      use_binary_mode
    end
  end

  # ========================================================================= #
  # === set_run_mode
  #
  # This will set the run-mode. The run-mode can be either :standalone or
  # :connected.
  # ========================================================================= #
  def set_run_mode(
      i = :standalone
    )
    @run_mode = i
  end

  # ========================================================================= #
  # === toggle_modes
  # ========================================================================= #
  def toggle_modes
    if @mode == :ascii
      set_mode :binary
    else
      set_mode :ascii
    end
    e mode?.to_s
  end

  # ========================================================================= #
  # === use_ascii_mode
  # ========================================================================= #
  def use_ascii_mode(be_verbose = false)
    e 'Using ascii mode now.' if be_verbose
    set_mode :ascii
  end

  # ========================================================================= #
  # === mode?
  #
  # @mode is a Symbol, hence we will not apply .to_s on it.
  # ========================================================================= #
  def mode?
    return @mode
  end

  # ========================================================================= #
  # === set_passive
  # ========================================================================= #
  def set_passive(
      be_verbose = false
    )
    if be_verbose
      e 'Enabling passive mode for FTP.'
    end
    ftp?.passive = true
  end

end; end