#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The Readline component will belong to class FtpParadise::InteractiveFtp.
#
# It would  not make any sense to use it elsewhere within the FtpParadise
# project, as we don't have other interactive elements elsewhere.
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/readline.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp # === FtpParadise::Interactive

  begin
    require 'readline'
  rescue LoadError; end # Silent rescue.

  # ========================================================================= #
  # === use_readline?
  # ========================================================================= #
  def use_readline?
    @use_readline
  end; alias do_we_use_readline? use_readline? # === do_we_use_readline?

  # ========================================================================= #
  # === do_not_use_readline
  # ========================================================================= #
  def do_not_use_readline
    @use_readline = false
  end

  # ========================================================================= #
  # === enable_readline
  # ========================================================================= #
  def enable_readline
    @use_readline = true
    try_to_use_readline
  end

  # ========================================================================= #
  # === try_to_use_readline
  #
  # This method ought to be called before we invoke the method menu().
  # ========================================================================= #
  def try_to_use_readline
    if @use_readline
      begin
        require 'readline' # Using readline since August 2011.
        setup_readline_completion
      rescue LoadError
        # =================================================================== #
        # In this case the user can not use the readline-module.
        # =================================================================== #
        do_not_use_readline
      end
    end
  end; alias consider_using_the_readline_module try_to_use_readline # === consider_using_the_readline_module

  # ========================================================================= #
  # === setup_readline_completion
  #
  # Call this method to invoke completion support.
  # ========================================================================= #
  def setup_readline_completion
    Readline.completion_case_fold = true # Ignore case on tab-completion.
    Readline.completer_word_break_characters = "\n"
    Readline.completion_append_character = ' '
    # ======================================================================= #
    # Act on Readline's completion functionality here.         (readline tag)
    #
    # We will respond to these events specifically:
    #
    #   - rcd
    #   - cd
    #   - upload
    #   - download
    #
    # ======================================================================= #
    Readline.completion_proc = proc {|input|
      _ = []
      case input # case tag
      # ===================================================================== #
      # To test it, do:
      #   rcd AUS<TAB>
      # ===================================================================== #
      when /^rcd/,
           /^rcd /,
           /^rc/,
           'rchange_dir',
           'lcd',
           'remote_change_directory'
        result = @_.return_directories # This will return the remote directories only.
        _ << result.map {|entry| input+' '+entry}
      # ===================================================================== #
      # === cd
      # ===================================================================== #
      when /^cd/,/^cd / # Fetch only directories.
        _ << @_.return_local_directories
      when /^upload/    # Act on upload.
        return_local_files.each { |array| _ << array }
      when /^download/  # Act on download.
        @_.array_file_listing.each { |array| _ << array.first }
      else # Default.
        # _ << return_local_files # Disabled in June 2014 because it gives completions to things I did not ask for.
      end
      _ = _.flatten
      _ # Return it here finally.
    }
  end

end; end