#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/remove.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp

  # ========================================================================= #
  # === remove_local
  # ========================================================================= #
  def remove_local(i)
    delete(i)
  end

  # ========================================================================= #
  # === remove_remote_directory
  #
  # Use this method to remove a remote directory.
  # ========================================================================= #
  def remove_remote_directory(i)
    e "Removing remote directory `#{sdir(i)}` next."
    # ======================================================================= #
    # Delegate to ftp?. :force_remove so we remove even non-empty directories.
    # ======================================================================= #
    ftp?.remove_directory(i, :force_remove)
  end

  # ========================================================================= #
  # === remote_remove
  #
  # Use this if you wish to remove a remote file or a remote directory.
  # ========================================================================= #
  def remote_remove(i)
    if i.is_a? Array
      i.each {|entry| remote_remove(entry) }
    else
      i = i.to_s.strip # Added as of July 2014.
      i = File.basename(i) if i.include? '/'
      # ===================================================================== #
      # The user may input numbers. If the input is a number, we will try to
      # find the corresponding entry - but only if no remote directory or
      # remote file exists with that name. Some directories may be named,
      # e. g. "123456".
      # ===================================================================== #
      unless this_remote_entry_exists? i
        i = convert_number_to_file(i) if i =~ /^\d+$/
      end
      # ===================================================================== #
      # Next check for user input. If the user did input a '*', then we
      # will expand that.
      # ===================================================================== #
      if i.include?('*') or i.include?('*.')
        i = ftp?.array_file_listing.select {|entry|
          entry[0].include? i.delete('*')
        }
        i.reject! {|line|
          line == '.' or line == '..' # Reject '.' and '..' entries.
        }
        i.map! {|entry|
          File.basename(entry)
        } # ^^^ This is a bit hackish.
        remote_remove(i)
      else
        # =================================================================== #
        # We try to find out whether we have a directory or a file next.
        # =================================================================== #
        if is_dir?(i)
          remove_remote_directory(i)
        else # Assume it is a file. 
          remove_remote_file(i)
        end
      end
    end
  end

  # ========================================================================= #
  # === remove_remote_file
  #
  # Use this method to attempt remove a remote file.
  # ========================================================================= #
  def remove_remote_file(i)
    i[0,1] = ''.dup if i.start_with? '/' # We don't like starting "/" characters here.
    e "Removing remote file `#{sfile(i)}` next." # We remove the remote file here.
    begin
      result = ftp?.remove_file(i)
      e "Successfully removed remote file `#{sfile(result)}`."
    rescue Net::FTPPermError => error
      e 'Can not remove file `'+sfile(i)+'` as you are not '\
        'the owner. ('+simp('Net::FTPPermError')+')'
      pp error
    end
  end

end; end