#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/show.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp

  # ========================================================================= #
  # === show_remote_directories
  # ========================================================================= #
  def show_remote_directories
    e "The remote directories at #{sdir(return_remote_pwd)} are:"
    e
    ftp?.return_remote_directories.each_with_index {|dir, index|
      index += 1
      e index.to_s.rjust(3)+') '+sdir(dir)
    }
    e
  end

  # ========================================================================= #
  # === show_remote_files
  #
  # Use this method to show all remote files on a FTP site.
  # ========================================================================= #
  def show_remote_files(
      mode = nil
    ) # Show the content of the remote host.
    ftp?.show_remote_files
  end

  # ========================================================================= #
  # === show_hosts
  # ========================================================================= #
  def show_hosts
    e "The available hosts are:#{N}#{N}"
    ftp?.array_available_hosts.each {|host|
      e simp("  #{host}")
    }; e
  end

  # ========================================================================= #
  # === show_remote_listing
  #
  # This will display the remote file listing.
  # ========================================================================= #
  def show_remote_listing
    ftp?.show_remote_listing
  end

  # ========================================================================= #
  # === show_status
  # ========================================================================= #
  def show_status
    cliner {
      # e
      # show_help # <- Disabled as of May 2017.
      e
      report_where_we_are_connected_to
      report_remote_directory # Added Dec 2011.
      report_local_dir
      # debug_it
      e
    }
  end

  # ========================================================================= #
  # === show_title
  #
  # Use this method to give us the title
  # ========================================================================= #
  def show_title
    e return_title
  end

  # ========================================================================= #
  # === show_history
  #
  # This method can be used to display the input-history of the user.
  # ========================================================================= #
  def show_history # History tag.
    e "The history for #{sfancy(return_title)} is:"
    @history.each_with_index { |key, index|
      e '%3s' % (index+1).to_s+' '+key
    }
  end

  # ========================================================================= #
  # === show_which_files_we_did_download_so_far
  # ========================================================================= #
  def show_which_files_we_did_download_so_far
    if @array_downloaded_files.empty?
      e 'We did not yet download any file.'
    else
      e 'We downloaded these files so far:'
      pp @array_downloaded_files
    end
  end

  # ========================================================================= #
  # === show_ftp_docu
  #
  # This method will show where to find ruby's ftp docu.
  # ========================================================================= #
  def show_ftp_docu
    e
    e '  https://www.ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html'
    e
  end

  # ========================================================================= #
  # === show_ftp_yaml_file
  # ========================================================================= #
  def show_ftp_yaml_file
    e 'The yaml file in question can be found at:'
    e
    e "  #{sfile(FILE_ROEBE_FTP)}"
    e
  end

  # ========================================================================= #
  # === show_modtime_of
  # ========================================================================= #
  def show_modtime_of(i)
    e "The modification time of #{sfile(i)} is:"
    e
    e '  '+sfancy(ftp?.modification_time_of?(i))
    e
  end

  # ========================================================================= #
  # === display_configuration_options                       (show config tag)
  #
  # This method will show the configuration settings.
  #
  # You can invoke this in an interactive FTP session by doing this:
  #
  #   config?
  #
  # ========================================================================= #
  def display_configuration_options # Config tag.
    cliner {
      if @file.to_s.empty?
        e '@file is: unassigned as of yet'
      else
        e '@file is: '+sfancy(@file.to_s)
      end
      e '@local_directory is: '+sfancy(@local_directory.to_s)
      e '@splitted is: '+sfancy(@splitted.to_s)
      e 'Will we use the Readline module? '+
        sfancy(verbose_truth(use_readline?))
      e 'Will we show full names (with full path)? '+
         sfancy(verbose_truth(show_full_names?))
      e 'What is the base directory? '+
         sfancy(@config.base_directory.to_s)
    }
  end

  # ========================================================================= #
  # === show_available_connections
  # ========================================================================= #
  def show_available_connections(
      i = ARRAY_AVAILABLE_HOSTS
    )
    i.each_with_index {|entry, index|
      e "#{index} #{entry}"
    } if Object.const_defined? :RoebeFtpConstants
  end

  # ========================================================================= #
  # === show_www_component
  # ========================================================================= #
  def show_www_component
    e 'Make sure to visit:'
    path = '$RSRC/ftp_paradise/lib/ftp_paradise/www/sinatra_web_interface.rb'
    e "  #{sfancy(path)}"
    _ = get_env_of("ruby #{path}")
    esystem _
  end

end; end