#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/upload.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp # === FtpParadise::Interactive

  # ========================================================================= #
  # === upload                                                 (upload tag)
  #
  # Use this method to upload something locally to the remote site.
  #
  # You can also upload several files in one go - to do this, try
  # one of these ways:
  #
  #   upload 7,8 # This would upload file position 7 and 8.
  #   upload 1..293
  #
  # ========================================================================= #
  def upload(
      i = Dir['*']
    )
    # ======================================================================= #
    # We will always work with an Array past the following point.
    # ======================================================================= #
    i = [i].flatten.compact # Safeguard against nil.
    if i.empty?
      i << @file if @file
    end
    i.map! {|entry|
      if entry.include?('-') and !entry.include?('.') # Assume a range was given like 1-20.
        splitted = entry.split('-')
        entry = (splitted[0] .. splitted[1]).to_a
      elsif entry.include?('..') # Assume a range such as: upload 1..25
        entry =~ /(\d+)\.\.(\d+)/
        start_position = $1.to_s.dup.to_i 
        end_position   = $2.to_s.dup.to_i
        entry = return_local_files[start_position, end_position]
      elsif entry.include?(',')
        splitted = i.split(',')
        entry = (splitted[0] .. splitted[1]).to_a
      elsif (entry == 'ALL') or (entry == '*')
        entry = Dir['*']
      elsif entry.include?('*.') or entry.include?('*')
        entry = Dir[entry]
      end
      entry
    }
    i.flatten!
    i.compact!
    if on_roebe? and !is_connected?
      connect_to :shevy
    end
    i.each {|this_file|
      if this_file =~ /^\d+$/ # upload only a specific number.
        sorted = Dir['*'].sort
        if i.is_a? Array
          i = i.first
        end
        this_file = sorted[i.to_i - 1]
      end
      _ = this_file # default to upload to 'all'
      if File.exist?(_)
        # =================================================================== #
        # Next, some files will always be in binary mode. These are
        # defined in the Array ARRAY_BINARY_FILES.
        # =================================================================== #
        if ARRAY_BINARY_FILES.include?(File.extname(_).delete('.')) and
          (@mode != :binary)
          use_binary_mode
        end
        case @mode
        when :ascii
          set_file(_)
          ftp?.upload(_) # And now, call the functionality in the FTP lib.
        else
          ftp?.upload_binary(_)
        end
      else
        opne swarn('But the file `')+
             sfile(i)+
             swarn('` does not exist.')
      end
    }
  end

end; end