#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/user_input.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp # === FtpParadise::InteractiveFtp

  # ========================================================================= #
  # === obtain_user_input
  #  
  # This method will obtain user input, and assign it to the variable
  # @user_input.
  # ========================================================================= #
  def obtain_user_input
    if @use_readline
      user_input = Readline.readline('', true)
      Readline::HISTORY.pop if user_input.strip.empty?
    else
      user_input = $stdin.gets.chomp
    end
    # ======================================================================= #
    # Get rid of potential '#' comments in the input.
    # ======================================================================= #
    if user_input.include? '#'
      user_input = user_input[0 .. user_input.index('#')-1]
    end
    user_input.strip! # Get rid of ' ' empty characters.
    _ = user_input
    unless _.empty? # append to history.
      try_to_append_this_to_the_history(_)
    end
    # ======================================================================= #
    # If the user did input a ';' character then we will assume that he
    # wants to send multiple commands. We will however NOT assume so
    # if the string also includes a 'http://' - in that case, we assume
    # a HTTP URL instead.
    # ======================================================================= #
    if user_input.include? ';' and !user_input.include?('http://')
      user_input = user_input.split(';')
    else
      user_input = [user_input] # In this case simply turn it into an Array. 
    end
    # ======================================================================= #
    # The @user_input variable should always be an Array.
    # ======================================================================= #
    @user_input = user_input
  end; alias get_user_input   obtain_user_input # === get_user_input
       alias fetch_user_input obtain_user_input # === fetch_user_input

end; end