#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/toplevel_methods/upload_and_download.rb'
# FtpParadise.download
# =========================================================================== #
module FtpParadise

  require 'ftp_paradise/toplevel_methods/e.rb'
  require 'ftp_paradise/toplevel_methods/opn.rb'

  # ========================================================================= #
  # === FtpParadise.upload_this_binary_file
  #
  # This method shall allow you to upload a binary file to a remote host.
  #
  # For now, this has my FTP information hardcoded.
  #
  # Usage examples:
  #
  #   FtpParadise.upload_this_binary_file("/home/x/src/htop/htop-2.0.2.tar.xz")
  #   FtpParadise.upload_this_binary_file("/home/x/src/htop/htop-2.0.2.tar.xz") {{ cd_into_this_directory: 'htdocs/' }}
  #   FtpParadise.upload_this_binary_file("/home/x/src/htop/htop-2.0.2.tar.xz") {{ cd_into_this_directory: 'www/' }}
  #   upload_this_binary_file /home/x/src/htop/htop-2.0.2.tar.xz
  #   upload_this_binary_file /home/x/src/htop/htop-2.0.2.tar.xz
  #
  # ========================================================================= #
  def self.upload_this_binary_file(
      file,
      be_verbose             = :be_silent,
      use_this_as_ftp_object = nil
    )
    yielded = nil
    yielded = yield if block_given?
    case yielded
    when :be_quiet
      be_verbose = false
    end
    case be_verbose
    when :be_silent
      be_verbose = false
    end
    unless use_this_as_ftp_object
      require 'ftp_paradise/connection/connection.rb'
      use_this_as_ftp_object = ::FtpParadise::Connection.new(:dont_run_yet) { :default_dataset }
      if be_verbose
        use_this_as_ftp_object.be_verbose
      else
        use_this_as_ftp_object.be_quiet
      end
      use_this_as_ftp_object.set_binary_mode
    end
    if yielded
      if yielded.is_a? Hash
        if yielded.has_key? :cd_into_this_directory
          use_this_as_ftp_object.remote_cd(yielded[:cd_into_this_directory])
        end
      end
    end
    use_this_as_ftp_object.upload(file, be_verbose) # Here, do the FTP upload finally. 
  end

  # ========================================================================= #
  # === FtpParadise.download
  # ========================================================================= #
  def self.download(
      this_remote_file,
      use_this_ftp_object
    )
    use_this_ftp_object.gettextfile(
      this_remote_file
    )
  end

  # ========================================================================= #
  # === FtpParadise.upload
  #
  # This toplevel-method can be used to upload something to a remote
  # target.
  #
  # The second argument called `to` will tell us where to upload to.
  # ========================================================================= #
  def self.upload(
      local_target,
      remote_target       = :short_variant,
      use_this_ftp_object = :shevy,
      &block
    )
    case remote_target
    when :short_variant
      remote_target = File.basename(local_target)
    end
    if use_this_ftp_object.is_a? Symbol
      require 'ftp_paradise/connection/connection.rb'
      use_this_ftp_object = FtpParadise::Connection.new(:dont_run_yet)
      opn; e "We will now connect to #{sfancy(use_this_ftp_object.to_s)}."
      use_this_ftp_object.connect_to
      use_this_ftp_object.upload(local_target)
    end
    use_this_ftp_object.put(
      local_target,
      remotefile = File.basename(localfile),
      &block
    )
  end

end